ROMPAGE: EQU	017
RAMPAGE: EQU	016
JMPADDR: EQU	374
RSAVADR: EQU	377	

CR:	EQU	015
PROMPT:	EQU	076
DASH:	EQU	055
COLON:	EQU	072
SPACE:	EQU	040
ZERO:	EQU	060
EIGHT:	EQU	070

KEYIN:  EQU 	004
KEYOUT: EQU     014

OUTPORT: EQU 	017	; S3 

;ram layout

JMPINSTR: ORG	RAMPAGE#JMPADDR
REGSAVE:  ORG	RAMPAGE#RSAVADR

	ORG 	ROMPAGE#0
;
; get character from INPUT device and parse
; 260 - 267 = equal octal character - shift into LS 3 bits of reg B
; otherwise look for single character command (scan command table at end of listing)
;
START:
	LAI	103	; display current HL and contents
; it's a command	
CMD:
	CAL	CLROUT	; first clear buffer

; table search routine. start at H:L address
; and go to end of page.  each table entry has
; two words: character and lwo adr. if a char is
; found in table Z=0 and jmp to low adr in this
; page 
	LHI	\HB\TTBL ;loop pointers
	LLI	\LB\TTBL ;
CMD2:
	CPM	
	JTZ	CMDFND	; found match
	INL		; 
	INL
	JFZ	CMD2	; not found, continue searching

; illegal command falls through to here
ILLCMD:
NXTCMD:
	LAI	SPACE	; insert space
	CAL	OUTPUT	;
	LAI	PROMPT	; send prompt
	CAL	OUTPUT	;	
GETCHAR:
	INP	KEYIN	; read keyboard 
	NDA		; is a new character present
	JFS	GETCHAR	
	OUT	KEYOUT	; ack character read
	NDI	177	; clear MSB
	CAL	OUTPUT	;
	CPI	EIGHT
	JFS	CMD

; It's a number - cycle through buffer
NUM:
	CPI	ZERO	; is it less than 0?
	JTS	ILLCMD	; yes, then is an ASCII control char
	NDI	007
	LCA
	LAB		; get current data from buffer
	RLC
	RLC		; shift left 3 bits
	RLC
	NDI	370	; clear low 3 bits
	ORC		; or in 3 new bits
	LBA		; put back in buffer
	JMP	GETCHAR	; continue without PROMPT
EX:
	LAE
	LCD
	JMP	JMPIND  ; execute it


; found valid command, jump to execute it
CMDFND:
	INL
	LAM		; get vector for L
	LCH		; vector for H is ROMPAGE (already in H)
JMPIND:
	LLI	\LB\JMPINSTR ; construct JMP instruction at
	LHI	\HB\JMPINSTR ; RAMPAGE
	LMI	104	; insert JMP op code
	INL
	LMA		; save vector (L is cached in E)
	INL
	LMC		; high address (H is cached in D)
	JMP	JMPINSTR ; execute it
;
; Command X - execute routine at address HL
; user routine call: all registers could be in play
CX:
	JMP	EX	
;
; Command H - copy BUFFER to H
CH:	LDB		; put B into H (H is cached in D)
	JMP	DHL	; and display current value of HL
;
; Command L - copy BUFFER to L
CL:	LEB		; Buffer B into L  (L is cached in E)
	JMP	DHL	; DUMP PTR and CONTENTS
;
; Command W - write BUFFER through HL to memory
; auto increment and display next location
CW:	LHD		; ptr to memory (H is cached in D)
	LLE		; ptr to memory (L is cached in E)
	LMB		; write B at address in memory
;
; Command I - increment HL 
CI:	INE		; increment L
	JFZ 	DHL	; no wrap, display memory	
	IND		; if L=0 increment H
	JMP 	DHL	; continue to display memory
;
; Command D - decrement HL
; destroys A
CD:	LAE		; save in A, (use to check for wrap)
	DCE		; decrement L (cached in E)
	NDA		; check for wrap
	JFZ 	DHL	; no wrap, now display memory
	DCD		; wrap, decrement H (cached in D)
;
; Command C - current memory contents  to serial port
; destroys D (= A), E
CC:	
DHL:
	LAI	SPACE	; insert space
	CAL	OUTPUT	;
	LAD		; H is cached in D
	CAL	POCTAL	; display H
	LAI	DASH	; dash between high and low
	CAL	OUTPUT	;
	LAE		; L is cached in E
	CAL	POCTAL	; display L
	LAI	COLON	; colon between address and data
	CAL	OUTPUT
	LHD		; move ptr to HL
	LLE
	LAM		; display contents
	CAL	POCTAL	; print octal
	JMP	NXTCMD	; get next command
;
; print octal number
; call this routine, don't jump to it
;
; destroys A (mangled), H (= RAMPAGE), L (= REGSAVE)
;
POCTAL:
	LHI	\HB\REGSAVE ; ptr to reg save area
	LLI	\LB\REGSAVE	
	LMA		; save input char
	RLC
	RLC
	NDI	003
	CAL	POCTAL2	; display MSB 2 bits
	LLI	\LB\REGSAVE ; ptr to reg save area
	LAM		; restore original char
	RAR
	RAR
	RAR
	CAL	POCTAL2	; display middle 3 bits
	LLI	\LB\REGSAVE ; ptr to reg save area
	LAM		; restore original char
POCTAL2:
	NDI	007	; mask 3 bits
	ORI	ZERO	; make ASCII
	JMP 	OUTPUT	; send it

; clear output (memory and shift register)
; destroys C (=A),  H(=RAMPAGE), L(=0)
; CURPTR - output ptr is initialized
; initialize scopewriter output flip flops
;
CLROUT:
	LCA			; save A register

	LAI 377
	OUT OUTPORT		; RESET PTR TO BEGINNING
	XRA			; clear A
	OUT OUTPORT		; 


        LHI 000			; space to every place 256 bytes
CONT:	LAI 240			; space char
	OUT OUTPORT
        NDI 177
        OUT OUTPORT
        INH
        LAH
        CPI 000
        JFZ CONT

	LAC			; restore A
	RET
             
;
; HERE IS THE USER DEFINED PRINT ROUTINE FOR SCOPEWRITER OUTPUT
; input:A, H must = RAMPAGE
; destroys C (= A), L(=CURPTR)
; 
;
OUTPUT:
	LCA
        ORI 200
	OUT OUTPORT
        NDI 177
        OUT OUTPORT
     
	LAC			; restore A (newest char) from "C"
	RET

;
; this table has octal charctets, control
; characters and addres of control subroutine
; note that first tabel entry is also ret
; above, I E overlap is OK
 	ORG	ROMPAGE#362
TTBL:
	DATA	'L'
	DATA	\LB\CL	; put buffer in low adr
	DATA	'H'	; H
	DATA	\LB\CH	; put buffer in high adr
	DATA	'I'	; I
	DATA	\LB\CI	; increment adr pointer
	DATA	'W'	; M
	DATA	\LB\CW	; write buffer in memory
	DATA	'X'	; X
	DATA	\LB\CX	; execute loaded program
	DATA	'D'	; D
	DATA	\LB\CD	; decrement address pointer
	DATA	'C'	; C
	DATA	\LB\CC	; display memory on panel
